#--
#
#################################################################################
# Copyright 2011 CENATIC y IEPALA
#
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas por la Comisión Europea– 
# versiones posteriores de la EUPL (la «Licencia»);
# Solo podrá usarse esta obra si se respeta la Licencia.
# Puede obtenerse una copia de la Licencia en:
#
# http://www.osor.eu/eupl/european-union-public-licence-eupl-v.1.1
#
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye «TAL CUAL»,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones que establece la Licencia.
#################################################################################
#
#++
# etapa

class Etapa < ActiveRecord::Base

  untranslate_all

  belongs_to :proyecto
  belongs_to :agente
  has_many :actividad_x_etapa
  has_many :actividad, :through => :actividad_x_etapa, :order => :codigo
  has_many :presupuesto
  has_many :tasa_cambio, :dependent => :destroy

  validates_presence_of :nombre, :message => _("Nombre") + _(" no pueden estar vacío.")
  #validates_presence_of [:fecha_inicio, :fecha_fin], :message => _("Nombre") + _(" no pueden estar vacío.")
  validates_uniqueness_of :nombre, :scope => [:proyecto_id, :agente_id], :message => _("Nombre") + _(" repetido.")
  validates_format_of :fecha_inicio, :with => /\d{4}-\d{1,2}-\d{1,2}/, :on => :create
  validates_format_of :fecha_fin, :with => /\d{4}-\d{1,2}-\d{1,2}/, :on => :create
  validate :comprobar_fechas

  # Antes de guardar convertimos la fecha de fin para que sea el ultimo dia
  #before_save  "self.fecha_fin = (self.fecha_fin >> 1) - 1"

  after_save :actualiza_tasa_cambio_moneda_principal
  before_destroy :verificar_borrado

  # Genera las tasas de cambio para la moneda principal en la etapa
  def actualiza_tasa_cambio_moneda_principal
    objeto = (self.proyecto || self.agente)
    # Si es una etapa nueva o se modifican las fechas, manda actualizar/crear tasas de cambio
    if self.id_was.nil? || self.fecha_inicio_changed? || self.fecha_fin_changed?
      TasaCambio.etapa_cambia_fechas(self)
    end
  end

	# Devuelve el numero de meses enteros que tiene el proyecto
  def meses
    # Calculamos el numero de meses sin contar los dias
    meses = ((self.fecha_fin.month) - self.fecha_inicio.month) + (12 * (self.fecha_fin.year - self.fecha_inicio.year))
    # Si la fecha de inicio es dia 1 y la de fin el final de mes, entonces hay un mes mas (el mes está completo)
    meses += 1 if self.fecha_inicio.day == 1 and (self.fecha_fin + 1.day).day == 1
    return meses
  end
	# Devuelve el numero de periodos mensuales (incluyendo mes no entero) que tiene el proyecto
  def periodos
    return ((self.fecha_fin.month) - self.fecha_inicio.month) + (12 * (self.fecha_fin.year - self.fecha_inicio.year)) + 1
  end

  # Comprueba que las fechas son correctas.
  def comprobar_fechas

    if self.fecha_fin and self.fecha_inicio
      # Comprueba que la fecha de inicio sea anterior a la de fin
      errors.add("fecha_fin", _("Fecha fin tiene que ser mayor que fecha inicio")) if self.fecha_fin <= self.fecha_inicio
      # Comprueba que las fechas no se solapen con otra Etapa
      (self.proyecto || self.agente).etapa.each do |et|
        if et.id != self.id && (
            (self.fecha_inicio >= et.fecha_inicio && self.fecha_inicio <= et.fecha_fin) ||
            (self.fecha_fin >= et.fecha_inicio && self.fecha_fin <= et.fecha_fin) ||
            (self.fecha_inicio <= et.fecha_inicio && self.fecha_fin >= et.fecha_fin) )
          errors.add("fecha", _("Las fechas se solapan con las de otra etapa"))
      end
    end

    else
     errors.add("fecha", _("fecha no puede esta vacio") ) 
    end
    # Para Convenios comprueba que todos los pacs tengan sus etapas dentro del pac
    self.proyecto.pacs.each do |pac|
      unless pac.etapa.empty?
        errors.add("fecha", _("Las fechas excluyen las definidas para la PAC") + " '" + pac.nombre + "'") if fecha_inicio > pac.fecha_de_inicio || fecha_fin < pac.fecha_de_fin
      end
    end if self.proyecto
    # Para PACs
    if self.proyecto && self.proyecto.convenio
      # Comprueba que las fechas esten entre las del convenio
      errors.add("fecha", _("Las fechas están fuera del Convenio")) if proyecto.convenio.fecha_de_inicio > fecha_inicio || proyecto.convenio.fecha_de_fin < fecha_fin
      # Comprueba que no haya otra PAC con esas fechas
      proyecto.convenio.pacs.each do |pac|
        unless pac.etapa.empty? || pac == self.proyecto
          errors.add("fecha", _("Las fechas se solapan con las de la PAC") + " '" + pac.nombre + "'") unless fecha_fin < pac.fecha_de_inicio || fecha_inicio > pac.fecha_de_fin 
        end
      end
    end
  end

  def etapa_siguiente
    return (self.proyecto || self.agente).etapa.first(:conditions => ["fecha_inicio > ?",self.fecha_fin], :order => "fecha_inicio")
  end

  def etapa_anterior
    return (self.proyecto || self.agente).etapa.last(:conditions => ["fecha_fin < ?",self.fecha_inicio], :order => "fecha_fin")
  end

  def verificar_borrado
    errors.add("Presupuesto", _("La etapa no se puede borrar. Tiene asociada presupuestos")) unless self.presupuesto.empty? 
    errors.add("Actividad", _("La etapa no se puede borrar. Tiene asociada actividades")) unless self.actividad.empty?
    return false unless errors.empty?
  end

end
